<?php

namespace smusatov\bzb2u\Render\Text;

class Line
{
    protected string $text;
    protected int $width;
    protected int $height;
    protected \ImagickDraw $draw;

    public function __construct(string $text)
    {
        if(empty(trim($text))) {
            throw new \Exception('Text in line not be empty');
        }

        $this->text = $text;

        $this->draw = new \ImagickDraw();
        $this->draw->setGravity(\Imagick::GRAVITY_CENTER);
        $this->draw->setStrokeAntialias(false);
        $this->draw->setTextAntialias(false);
        $this->draw->setFillColor(new \ImagickPixel('#000000'));
        $this->draw->setTextUnderColor(new \ImagickPixel('#ffffff'));

        $this->setFontRegular();
    }

    public function setFontBold()
    {
        $this->draw->setFont('./font/Bold.ttf');
    }

    public function setFontRegular()
    {
        $this->draw->setFont('./font/Regular.ttf');
    }

    public function setFontThin()
    {
        $this->draw->setFont('./font/Thin.ttf');
    }

    public function setSize(int $width, int $height)
    {
        if($width <= 0) {
            throw new \Exception('Max width must be greater than 0');
        }

        if($height <= 0) {
            throw new \Exception('Max height must be greater than 0');
        }

        $this->width = $width;
        $this->height = $height;
    }

    public function render(): \Imagick
    {
        $im = new \Imagick();
        $im->setGravity(\Imagick::GRAVITY_CENTER);
        $im->setType(\Imagick::IMGTYPE_BILEVEL);
        $im->setAntiAlias(false);
        $im->setFormat('png');
        $im->setCompression(\Imagick::COMPRESSION_NO);

        $fontSize = 7;
        do {
            $fontSize++;
            $this->draw->setFontSize($fontSize);
            $metrics = $im->queryFontMetrics($this->draw, $this->text);
        } while($metrics['textWidth'] <= $this->width && $metrics['textHeight'] <= $this->height);
        $this->draw->setFontSize($fontSize - 1);

        $im->newImage($this->width, $this->height, new \ImagickPixel('#ffffff'), 'png');
        $im->annotateImage($this->draw, 0, 0, 0, $this->text);

        return $im;
    }
}
